"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("mz/fs");
const path = require("path");
const glob = require("glob");
const iterare_1 = require("iterare");
const opentracing_1 = require("opentracing");
const semaphore_async_await_1 = require("semaphore-async-await");
const util_1 = require("./util");
class RemoteFileSystem {
    constructor(client) {
        this.client = client;
    }
    /**
     * The files request is sent from the server to the client to request a list of all files in the workspace or inside the directory of the base parameter, if given.
     * A language server can use the result to index files by filtering and doing a content request for each text document of interest.
     */
    getWorkspaceFiles(base) {
        return __awaiter(this, void 0, void 0, function* () {
            return iterare_1.default(yield this.client.workspaceXfiles({ base }))
                .map(textDocument => textDocument.uri);
        });
    }
    /**
     * The content request is sent from the server to the client to request the current content of any text document. This allows language servers to operate without accessing the file system directly.
     */
    getTextDocumentContent(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            const textDocument = yield this.client.textDocumentXcontent({ textDocument: { uri } });
            return textDocument.text;
        });
    }
}
exports.RemoteFileSystem = RemoteFileSystem;
class LocalFileSystem {
    /**
     * @param rootPath The root directory path that relative URIs should be resolved to
     */
    constructor(rootPath) {
        this.rootPath = rootPath;
    }
    /**
     * Converts the URI to an absolute path
     */
    resolveUriToPath(uri) {
        return util_1.toUnixPath(path.resolve(this.rootPath, util_1.uri2path(uri)));
    }
    getWorkspaceFiles(base) {
        return __awaiter(this, void 0, void 0, function* () {
            // Even if no base provided, still need to call resolveUriToPath which may be overridden
            const root = this.resolveUriToPath(base || util_1.path2uri('', this.rootPath));
            const baseUri = util_1.path2uri('', util_1.normalizeDir(root)) + '/';
            const files = yield new Promise((resolve, reject) => {
                glob('*', { cwd: root, nodir: true, matchBase: true }, (err, matches) => err ? reject(err) : resolve(matches));
            });
            return iterare_1.default(files).map(file => baseUri + file.split('/').map(encodeURIComponent).join('/'));
        });
    }
    getTextDocumentContent(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            return fs.readFile(this.resolveUriToPath(uri), 'utf8');
        });
    }
}
exports.LocalFileSystem = LocalFileSystem;
/**
 * Synchronizes a remote file system to an in-memory file system
 *
 * TODO: Implement Disposable with Disposer
 */
class FileSystemUpdater {
    constructor(remoteFs, inMemoryFs) {
        this.remoteFs = remoteFs;
        this.inMemoryFs = inMemoryFs;
        /**
         * Map from URI to Promise of pending or fulfilled content fetch
         */
        this.fetches = new Map();
        /**
         * Limits concurrent fetches to not fetch thousands of files in parallel
         */
        this.concurrencyLimit = new semaphore_async_await_1.default(100);
    }
    /**
     * Fetches the file content for the given URI and adds the content to the in-memory file system
     *
     * @param uri URI of the file to fetch
     * @param childOf A parent span for tracing
     */
    fetch(uri, childOf = new opentracing_1.Span()) {
        return __awaiter(this, void 0, void 0, function* () {
            // Limit concurrent fetches
            const promise = this.concurrencyLimit.execute(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    const content = yield this.remoteFs.getTextDocumentContent(uri);
                    this.inMemoryFs.add(uri, content);
                    this.inMemoryFs.getContent(uri);
                }
                catch (err) {
                    this.fetches.delete(uri);
                    throw err;
                }
            }));
            this.fetches.set(uri, promise);
            return promise;
        });
    }
    /**
     * Returns a promise that is resolved when the given URI has been fetched (at least once) to the in-memory file system.
     * This function cannot be cancelled because multiple callers get the result of the same operation.
     *
     * @param uri URI of the file to ensure
     * @param childOf A parent span for tracing
     */
    ensure(uri, childOf = new opentracing_1.Span()) {
        return __awaiter(this, void 0, void 0, function* () {
            const span = childOf.tracer().startSpan('Ensure file content', { childOf });
            span.addTags({ uri });
            try {
                return yield (this.fetches.get(uri) || this.fetch(uri, span));
            }
            catch (err) {
                span.setTag('error', true);
                span.log({ 'event': 'error', 'error.object': err });
                throw err;
            }
            finally {
                span.finish();
            }
        });
    }
    /**
     * Fetches the file/directory structure for the given directory from the remote file system and saves it in the in-memory file system
     *
     * @param childOf A parent span for tracing
     */
    fetchStructure(childOf = new opentracing_1.Span()) {
        const promise = (() => __awaiter(this, void 0, void 0, function* () {
            const span = childOf.tracer().startSpan('Fetch workspace structure', { childOf });
            try {
                const uris = yield this.remoteFs.getWorkspaceFiles();
                for (const uri of uris) {
                    this.inMemoryFs.add(uri);
                }
            }
            catch (err) {
                this.structureFetch = undefined;
                span.setTag('error', true);
                span.log({ 'event': 'error', 'error.object': err });
                throw err;
            }
            finally {
                span.finish();
            }
        }))();
        this.structureFetch = promise;
        return promise;
    }
    /**
     * Returns a promise that is resolved as soon as the file/directory structure for the given directory has been synced
     * from the remote file system to the in-memory file system (at least once)
     *
     * @param childOf A parent span for tracing
     */
    ensureStructure(childOf = new opentracing_1.Span()) {
        return __awaiter(this, void 0, void 0, function* () {
            const span = childOf.tracer().startSpan('Ensure workspace structure', { childOf });
            try {
                return yield (this.structureFetch || this.fetchStructure(span));
            }
            catch (err) {
                span.setTag('error', true);
                span.log({ 'event': 'error', 'error.object': err });
                throw err;
            }
            finally {
                span.finish();
            }
        });
    }
    /**
     * Invalidates the structure fetch cache.
     * The next call to `ensureStructure` will do a refetch.
     */
    invalidateStructure() {
        this.structureFetch = undefined;
    }
}
exports.FileSystemUpdater = FileSystemUpdater;
//# sourceMappingURL=fs.js.map