"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("@reactivex/rxjs");
const events_1 = require("events");
const opentracing_1 = require("opentracing");
const sinon = require("sinon");
const vscode_jsonrpc_1 = require("vscode-jsonrpc");
const vscode_jsonrpc_2 = require("vscode-jsonrpc");
const connection_1 = require("../connection");
const typescript_service_1 = require("../typescript-service");
describe('connection', () => {
    describe('registerLanguageHandler()', () => {
        it('should return MethodNotFound error when the method does not exist on handler', () => __awaiter(this, void 0, void 0, function* () {
            const handler = Object.create(typescript_service_1.TypeScriptService.prototype);
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'whatever', params });
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({ jsonrpc: '2.0', id: 1, error: { code: vscode_jsonrpc_2.ErrorCodes.MethodNotFound } }));
        }));
        it('should return MethodNotFound error when the method is prefixed with an underscore', () => __awaiter(this, void 0, void 0, function* () {
            const handler = { _privateMethod: sinon.spy() };
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'textDocument/hover', params });
            sinon.assert.notCalled(handler._privateMethod);
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({ jsonrpc: '2.0', id: 1, error: { code: vscode_jsonrpc_2.ErrorCodes.MethodNotFound } }));
        }));
        it('should call a handler on request and send the result of the returned Promise', () => __awaiter(this, void 0, void 0, function* () {
            const handler = Object.create(typescript_service_1.TypeScriptService.prototype);
            const hoverStub = sinon.stub(handler, 'textDocumentHover').returns(Promise.resolve(2));
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'textDocument/hover', params });
            sinon.assert.calledOnce(hoverStub);
            sinon.assert.calledWithExactly(hoverStub, params, sinon.match.instanceOf(opentracing_1.Span));
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({ jsonrpc: '2.0', id: 1, result: 2 }));
        }));
        it('should call a handler on request and send the result of the returned Observable', () => __awaiter(this, void 0, void 0, function* () {
            const handler = Object.create(typescript_service_1.TypeScriptService.prototype);
            const hoverStub = sinon.stub(handler, 'textDocumentHover').returns(rxjs_1.Observable.of(2));
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'textDocument/hover', params });
            sinon.assert.calledOnce(hoverStub);
            sinon.assert.calledWithExactly(hoverStub, params, sinon.match.instanceOf(opentracing_1.Span));
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({ jsonrpc: '2.0', id: 1, result: 2 }));
        }));
        it('should call a handler on request and send the thrown error of the returned Observable', () => __awaiter(this, void 0, void 0, function* () {
            const handler = Object.create(typescript_service_1.TypeScriptService.prototype);
            const hoverStub = sinon.stub(handler, 'textDocumentHover').returns(rxjs_1.Observable.throw(Object.assign(new Error('Something happened'), {
                code: vscode_jsonrpc_2.ErrorCodes.serverErrorStart,
                whatever: 123
            })));
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'textDocument/hover', params });
            sinon.assert.calledOnce(hoverStub);
            sinon.assert.calledWithExactly(hoverStub, params, sinon.match.instanceOf(opentracing_1.Span));
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({
                jsonrpc: '2.0',
                id: 1,
                error: {
                    message: 'Something happened',
                    code: vscode_jsonrpc_2.ErrorCodes.serverErrorStart,
                    data: { whatever: 123 }
                }
            }));
        }));
        it('should call a handler on request and send the returned synchronous value', () => __awaiter(this, void 0, void 0, function* () {
            const handler = Object.create(typescript_service_1.TypeScriptService.prototype);
            const hoverStub = sinon.stub(handler, 'textDocumentHover').returns(2);
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'textDocument/hover', params });
            sinon.assert.calledOnce(hoverStub);
            sinon.assert.calledWithExactly(hoverStub, params, sinon.match.instanceOf(opentracing_1.Span));
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({ jsonrpc: '2.0', id: 1, result: 2 }));
        }));
        it('should call a handler on request and send the result of the returned Observable', () => __awaiter(this, void 0, void 0, function* () {
            const handler = Object.create(typescript_service_1.TypeScriptService.prototype);
            const hoverStub = sinon.stub(handler, 'textDocumentHover').returns(rxjs_1.Observable.of(2));
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'textDocument/hover', params });
            sinon.assert.calledOnce(hoverStub);
            sinon.assert.calledWithExactly(hoverStub, params, sinon.match.instanceOf(opentracing_1.Span));
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({ jsonrpc: '2.0', id: 1, result: 2 }));
        }));
        it('should unsubscribe from the returned Observable when $/cancelRequest was sent and return a RequestCancelled error', () => __awaiter(this, void 0, void 0, function* () {
            const handler = Object.create(typescript_service_1.TypeScriptService.prototype);
            const unsubscribeHandler = sinon.spy();
            const hoverStub = sinon.stub(handler, 'textDocumentHover').returns(new rxjs_1.Observable(subscriber => unsubscribeHandler));
            const emitter = new events_1.EventEmitter();
            const writer = {
                write: sinon.spy(),
                onError: vscode_jsonrpc_1.Event.None,
                onClose: vscode_jsonrpc_1.Event.None
            };
            connection_1.registerLanguageHandler(emitter, writer, handler);
            const params = [1, 1];
            emitter.emit('message', { jsonrpc: '2.0', id: 1, method: 'textDocument/hover', params });
            sinon.assert.calledOnce(hoverStub);
            sinon.assert.calledWithExactly(hoverStub, params, sinon.match.instanceOf(opentracing_1.Span));
            emitter.emit('message', { jsonrpc: '2.0', method: '$/cancelRequest', params: { id: 1 } });
            yield new Promise(resolve => setTimeout(resolve, 0));
            sinon.assert.calledOnce(unsubscribeHandler);
            sinon.assert.calledOnce(writer.write);
            sinon.assert.calledWithExactly(writer.write, sinon.match({ jsonrpc: '2.0', id: 1, error: { code: vscode_jsonrpc_2.ErrorCodes.RequestCancelled } }));
        }));
    });
});
//# sourceMappingURL=connection.test.js.map