import { Span } from 'opentracing';
import { CompletionList, DidChangeTextDocumentParams, DidCloseTextDocumentParams, DidOpenTextDocumentParams, DidSaveTextDocumentParams, DocumentSymbolParams, Hover, InitializeResult, Location, ReferenceParams, SymbolInformation, TextDocumentPositionParams } from 'vscode-languageserver';
import { FileSystem, FileSystemUpdater } from './fs';
import { RemoteLanguageClient } from './lang-handler';
import { Logger } from './logging';
import * as pm from './project-manager';
import { DependencyReference, InitializeParams, PackageInformation, ReferenceInformation, SymbolLocationInformation, WorkspaceReferenceParams, WorkspaceSymbolParams } from './request-type';
export interface TypeScriptServiceOptions {
    traceModuleResolution?: boolean;
    strict?: boolean;
}
export declare type TypeScriptServiceFactory = (client: RemoteLanguageClient, options?: TypeScriptServiceOptions) => TypeScriptService;
/**
 * Handles incoming requests and return responses. There is a one-to-one-to-one
 * correspondence between TCP connection, TypeScriptService instance, and
 * language workspace. TypeScriptService caches data from the compiler across
 * requests. The lifetime of the TypeScriptService instance is tied to the
 * lifetime of the TCP connection, so its caches are deleted after the
 * connection is torn down.
 *
 * Methods are camelCase versions of the LSP spec methods and dynamically
 * dispatched. Methods not to be exposed over JSON RPC are prefixed with an
 * underscore.
 */
export declare class TypeScriptService {
    protected client: RemoteLanguageClient;
    protected options: TypeScriptServiceOptions;
    projectManager: pm.ProjectManager;
    /**
     * The rootPath as passed to `initialize` or converted from `rootUri`
     */
    root: string;
    /**
     * The root URI as passed to `initialize` or converted from `rootPath`
     */
    protected rootUri: string;
    private emptyQueryWorkspaceSymbols;
    private traceModuleResolution;
    /**
     * The remote (or local), asynchronous, file system to fetch files from
     */
    protected fileSystem: FileSystem;
    protected logger: Logger;
    /**
     * Holds file contents and workspace structure in memory
     */
    protected inMemoryFileSystem: pm.InMemoryFileSystem;
    /**
     * Syncs the remote file system with the in-memory file system
     */
    protected updater: FileSystemUpdater;
    /**
     * Set to true when shutdown is called
     */
    shutdownCalled: boolean;
    constructor(client: RemoteLanguageClient, options?: TypeScriptServiceOptions);
    initialize(params: InitializeParams, span?: Span): Promise<InitializeResult>;
    /**
     * Initializes the remote file system and in-memory file system.
     * Can be overridden
     *
     * @param accessDisk Whether the language server is allowed to access the local file system
     */
    protected _initializeFileSystems(accessDisk: boolean): void;
    shutdown(params?: {}, span?: Span): Promise<void>;
    textDocumentDefinition(params: TextDocumentPositionParams, span?: Span): Promise<Location[]>;
    textDocumentXdefinition(params: TextDocumentPositionParams, span?: Span): Promise<SymbolLocationInformation[]>;
    textDocumentHover(params: TextDocumentPositionParams, span?: Span): Promise<Hover>;
    textDocumentReferences(params: ReferenceParams, span?: Span): Promise<Location[]>;
    workspaceSymbol(params: WorkspaceSymbolParams, span?: Span): Promise<SymbolInformation[]>;
    protected _workspaceSymbolDefinitelyTyped(params: WorkspaceSymbolParams): Promise<SymbolInformation[] | null>;
    textDocumentDocumentSymbol(params: DocumentSymbolParams, span?: Span): Promise<SymbolInformation[]>;
    workspaceXreferences(params: WorkspaceReferenceParams, span?: Span): Promise<ReferenceInformation[]>;
    workspaceXpackages(params?: {}, span?: Span): Promise<PackageInformation[]>;
    workspaceXdependencies(params?: {}, span?: Span): Promise<DependencyReference[]>;
    textDocumentCompletion(params: TextDocumentPositionParams, span?: Span): Promise<CompletionList>;
    private _walkMostAST(node, visit);
    textDocumentDidOpen(params: DidOpenTextDocumentParams): Promise<void>;
    textDocumentDidChange(params: DidChangeTextDocumentParams): Promise<void>;
    textDocumentDidSave(params: DidSaveTextDocumentParams): Promise<void>;
    textDocumentDidClose(params: DidCloseTextDocumentParams): Promise<void>;
    /**
     * Fetches (or creates if needed) source file object for a given file name
     * @param configuration project configuration
     * @param fileName file name to fetch source file for or create it
     */
    private _getSourceFile(configuration, fileName);
    /**
     * Produces async function that converts ReferenceEntry object to Location
     */
    private _transformReference(root, program, includeDeclaration, ref);
    /**
     * transformNavItem transforms a NavigateToItem instance to a SymbolInformation instance
     */
    private _transformNavItem(root, program, item);
    private _collectWorkspaceSymbols(configs, query?, symQuery?);
    /**
     * Transforms definition's file name to URI. If definition belongs to TypeScript library,
     * returns git://github.com/Microsoft/TypeScript URL, otherwise returns file:// one
     */
    private _defUri(filePath);
    /**
     * Fetches up to limit navigation bar items from given project, flattens them
     */
    private _getNavigationTreeItems(configuration, limit?);
    /**
     * Flattens navigation tree by transforming it to one-dimensional array.
     * Some items (source files, modules) may be excluded
     */
    private _flattenNavigationTreeItem(item, parent, sourceFile, result, limit?);
    /**
     * Transforms NavigationTree to SymbolInformation
     */
    private _transformNavigationTreeItem(item, parent, sourceFile);
    /**
     * @return true if navigation tree item is acceptable for inclusion into workspace/symbols
     */
    private static isAcceptableNavigationTreeItem(item);
}
